#!/usr/bin/perl
#    Big Sister network monitor
#    Copyright (C) 1998-2002  Thomas Aeby
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

use strict;

$BigSister::common::Usage = "[-f feature1 ...] [-d device] [-t testname] (known|tests)";

@BigSister::common::options = ( "d=s", "f=s\@", "t=s" );
use lib "."; use lib "platforms"; 		#inslib
use BigSister::common;

proginit();

require Monitor::Tester;

my $mode = shift;
$mode = (($BigSister::common::opt_t || @BigSister::common::opt_f) ? "tests" : "known") unless( $mode );
my $device = $BigSister::common::opt_d;

my %types = (
    "kbsize" => "size in kBytes",
    "kbitrate" => "rate in kbit/s",
    "percentage" => "percentage",
    "int" => "integer number",
    "pattern" => "regular expression/string"
);

($mode eq "known") && known();
($mode eq "tests") && tests();


sub known {
    my( @tests ) = Monitor::Tester::testnames();
    my %devices = ();
    my %names = ();
    my %features = ();

    foreach my $testname (@tests) {
        my $test = Monitor::Tester::gettest( $testname );
	$test = newfrom Monitor::Tester( $test );
	next unless( $test );
	my $dev = $test->expand_value( "device" );
	next if( $device && ($dev ne $device) );
	$devices{$dev} = 1;
	$names{$test->expand_value( "name" )} = 1;
	foreach my $feature (@{$test->expand_values( "features" )}) {
	    $features{$feature} = 1;
	}
    }

    print "Device types:\n";
    print "============\n\n";
    fancy_print( %devices );

    print "\nFeature names:\n";
    print "=============\n\n";
    fancy_print( %features );

    print "\nTests:\n";
    print "=====\n\n";
    fancy_print( %names );
}


sub fancy_print {
    my( %list ) = @_;

    foreach my $name (sort keys %list) {
	 next unless( $name );
         print "  $name\n";
    }
} 



sub tests {
    my $selected_test = $BigSister::common::opt_t;
    my @features = @BigSister::common::opt_f;

    my $args = {
        "features" => join( ",", @features )
    };
    if( $device ) {
         $args->{"device"} = $device;
    }
    my %done = ();
    foreach my $testname (Monitor::Tester::testnames()) {
        my $test = newfrom Monitor::Tester( Monitor::Tester::gettest( $testname ) );
	next unless( $test );
	my $thisname = $test->expand_value( "name" );
	next if( $done{$thisname} || ($selected_test && ($thisname ne $selected_test)) );
	$done{$thisname} = 1;
	$args->{"name"} = $thisname;
	my $best = new Monitor::Tester( $args );
	next unless( $best );
	print $best->expand_value( "name" );
	print "\n\n";
	my $description = $best->expand_value( "description" );
	multiline_print( $description, "  ", 62 );
	print "\n";
	foreach my $arg ( @{$best->expand_values( "arguments")} ) {
	    arg_print( $arg );
        }
	print "\n---------------------------------------\n\n";
    }
} 



sub arg_print {
    my( $arg ) = @_;

    return unless( $arg =~ /^(.*?):(.*?):(.*)/ );
    my( $name, $type, $desc ) = ($1,$2,$3);
    print "  $name: ";
    my @types = split( /\|/, $type );
    foreach my $type (@types) {
        $type = expand_type( $type );
    }
    print join( " OR ", @types );
    print "\n";
    multiline_print( $desc, "    ", 60 );
}


sub expand_type {
    my( $type ) = @_;

    my $exp = "";
    if( $type =~ /^\@/ ) {
        $type = $';
	$exp = "list of ";
    }
    if( defined $types{$type} ) {
        return "$exp$types{$type}";
    }
    return $exp.$type;
}


sub multiline_print {
    my( $string, $prefix, $length ) = @_;

    while( length( $string ) > $length ) {
        my $max = $length;
        for( my $i=0; $i<=$length; $i++ ) {
	    if( substr( $string, $i, 1 ) =~ /[\s\t\-]/ ) {
	        $max = $i;
	    }
        }
	print $prefix.(substr( $string, 0, $max))."\n";
	$string = substr( $string, $max+1 );
	$string =~ s/^[\s\t]+//;
    }
    print "$prefix$string\n" if( $string  );
}
